use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct Doc {
    pub version: i32,
    pub content: Vec<DocNode>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
#[serde(tag = "type", content = "content")]
pub enum DocNode {
    Paragraph(Vec<DocTextNode>),
    BulletList(Vec<DocNode>),
    ListItem(Vec<DocNode>),
    OrderedList(Vec<DocNode>),
    Rule,
    Heading(Vec<DocTextNode>),
    MediaSingle(Vec<DocTextMarker>),
    Panel(Vec<DocNode>),
    CodeBlock(Vec<DocTextNode>),
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
#[serde(tag = "type")]
pub enum DocTextNode {
    Text(DocTextNodeContent),
    HardBreak,
    InlineCard(DocInlineCard),
    Mention(DocMention),
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct DocTextNodeContent {
    pub text: String,
    pub marks: Option<Vec<DocTextMarker>>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
#[serde(tag = "type", content = "attrs")]
pub enum DocTextMarker {
    Link(DocTextMarkerLink),
    Strong,
    Code,
    Media(DocMedia),
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct DocTextMarkerLink {
    pub href: String,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
#[serde(tag = "type")]
pub enum DocMedia {
    File(DocMediaFile),
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct DocMediaFile {
    pub id: String,
    pub alt: String,
    pub collection: String,
    pub height: i32,
    pub width: i32,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct DocInlineCard {
    pub attrs: DocInlineCardAttrs,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct DocInlineCardAttrs {
    pub url: String,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct DocMention {
    pub attrs: DocMentionAttrs,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct DocMentionAttrs {
    pub id: String,
    pub text: String,
}
